/* $Id: cpl_image_filter_body.h,v 1.35 2008/07/21 08:48:39 llundin Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2008 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Type dependent macros */
#if CPL_CLASS == CPL_CLASS_DOUBLE
#define CPL_TYPE double
#define CPL_TYPE_T CPL_TYPE_DOUBLE
#define CPL_IMAGE_GET_DATA cpl_image_get_data_double
#define CPL_IMAGE_GET_DATA_CONST cpl_image_get_data_double_const
#define CPL_TYPE_MEDIAN cpl_tools_get_median_double


#elif CPL_CLASS == CPL_CLASS_FLOAT
#define CPL_TYPE float
#define CPL_TYPE_T CPL_TYPE_FLOAT
#define CPL_IMAGE_GET_DATA cpl_image_get_data_float
#define CPL_IMAGE_GET_DATA_CONST cpl_image_get_data_float_const
#define CPL_TYPE_MEDIAN cpl_tools_get_median_float

#else
#undef CPL_TYPE
#undef CPL_TYPE_T
#endif

#if defined CPL_OPERATION && CPL_OPERATION == CPL_IMAGE_FILTER_LINEAR_NOBPM

        case CPL_TYPE_T:
        {
            CPL_TYPE * pi;
            CPL_TYPE * po;

            image_out = cpl_image_new(in->nx, in->ny, CPL_TYPE_T);
            pi = (CPL_TYPE*)in->pixels ;
            po = (CPL_TYPE*)image_out->pixels ;
            /* Main filter loop */
            for (j=hsy ; j<in->ny-hsy ; j++) {
                for (i=hsx ; i<in->nx-hsx ; i++) {
                    /* Curent pixel position */
                    curr_pos = i + j*in->nx ;
                    sum_pix = 0.0 ;
                    /* Go into upper left corner of current pixel   */
                    im_pos = curr_pos - hsx + hsy*in->nx ;
                    filt_pos = 0 ;
                    for (k=0 ; k<nr ; k++) {
                        for (l=0 ; l<nc ; l++) {
                            sum_pix += ker_arr[filt_pos] * (double)pi[im_pos] ;
                            /* Next col */
                            filt_pos++ ;
                            im_pos++ ;
                        }
                        /* Next row */
                        im_pos -= in->nx + nc ;
                    }
                    /* Normalize output */
                    sum_pix *= ker_norm ;
                    /* Assign value to image_out    */
                    po[curr_pos] = (CPL_TYPE)sum_pix ;
                }
            }
            break ;
          }

#elif defined CPL_OPERATION && CPL_OPERATION == CPL_IMAGE_FILTER_LINEAR

        case CPL_TYPE_T:
        {
            const CPL_TYPE * pi = (const CPL_TYPE*)in->pixels;
            CPL_TYPE       * po = (CPL_TYPE*)filtered->pixels;

            /* Main filter loop */
            for (j=0 ; j<in->ny ; j++) {
                for (i=0 ; i<in->nx ; i++) {
                    /* Curent pixel position */
                    curr_pos = i + j*in->nx ;
                    if (pbpm_dil[curr_pos] == CPL_BINARY_1) {
                        /* This pixel has to be recomputed */
                        ker_norm = 0.0 ;
                        sum_pix = 0.0 ;
                        /* Go into upper left corner of current pixel   */
                        x = i - hsx ;
                        y = j + hsy ;
                        filt_pos = 0 ;
                        for (k=0 ; k<nr ; k++) {
                            for (l=0 ; l<nc ; l++) {
                                im_pos = x + y * in->nx ;
                                if (x>=0 && x<in->nx && y>=0 && y<in->ny) {
                                    if (pbpm[im_pos] == CPL_BINARY_0) {
                                        /* This pixel is not bad any more */
                                        pbpm_dil[curr_pos] = CPL_BINARY_0 ; 
                                        sum_pix += ker_arr[filt_pos] * 
                                            (double)pi[im_pos] ;
                                        ker_norm += fabs(ker_arr[filt_pos]) ;
                                    }
                                }
                                /* Next col */
                                filt_pos++ ;
                                x++ ;
                            }
                            /* Next row */
                            x -= nc ;
                            y-- ;
                        }
                        /* Compute inverse sum of filters coeffs */
                        if (fabs(sum_pix) > FLT_MAX * ker_norm) {
                            /* Reject the pixel */
                            pbpm_dil[curr_pos] = CPL_BINARY_1;
                            po[curr_pos] = (CPL_TYPE)0 ;
                        } else {
                            /* Normalize output */
                            /* Assign value to image_out */
                            if (ker_norm == 0) po[curr_pos]=(CPL_TYPE)0 ;
                            else po[curr_pos] = (CPL_TYPE) (sum_pix / ker_norm);
                        }
                    }
                }
            }
            break ;
          }

#elif defined CPL_OPERATION && CPL_OPERATION == CPL_IMAGE_FILTER_MORPHO_NOBPM

        case CPL_TYPE_T:
        {
            CPL_TYPE * pi;
            CPL_TYPE * po;

            image_out = cpl_image_new(in->nx, in->ny, CPL_TYPE_T);
            pi = (CPL_TYPE*)in->pixels ;
            po = (CPL_TYPE*)image_out->pixels ;
            /* Main filter loop */
            for (j=hsy ; j<in->ny-hsy ; j++) {
                for (i=hsx ; i<in->nx-hsx ; i++) {
                    /* Curent pixel position */
                    curr_pos = i + j*in->nx ;
                    /* Go into upper left corner of current pixel   */
                    im_pos = curr_pos - hsx + hsy*in->nx ;
                    filt_pos = 0 ;
                    /* Store all relevant pixels in an array for sorting    */
                    for (k=0 ; k<nr ; k++) {
                        for (l=0 ; l<nc ; l++) {
                            pix_arr[filt_pos] = (double)pi[im_pos] ;
                            /* Next col */
                            filt_pos++ ; 
                            im_pos++ ; 
                        }
                        /* Next row */
                        im_pos -= in->nx + nc ; 
                    }
                    /* Sort array with standard sorting routine */
                    cpl_tools_sort_double(pix_arr, nr*nc) ;
                    sum_pix = 0.0 ;
                    for (k=0 ; k<nr*nc ; k++) sum_pix += ker_arr[k]*pix_arr[k];
                    /* Normalize output */
                    sum_pix *= ker_norm ;
                    /* Assign value to image_out */
                    po[curr_pos] = (CPL_TYPE)sum_pix ;
                }
            }
            break ;
          }

#elif defined CPL_OPERATION && CPL_OPERATION == CPL_IMAGE_FILTER_MORPHO

        case CPL_TYPE_T:
        {
            CPL_TYPE * pi;
            CPL_TYPE * po;

            pi = (CPL_TYPE*)in->pixels ;
            po = (CPL_TYPE*)filtered->pixels ;
            /* Main filter loop */
            for (j=0 ; j<in->ny ; j++) {
                for (i=0 ; i<in->nx ; i++) {
                    /* Curent pixel position */
                    curr_pos = i + j*in->nx ;
                    if (pbpm_dil[curr_pos] == CPL_BINARY_1) {
                        /* Go into upper left corner of current pixel   */
                        x = i - hsx ;
                        y = j + hsy ;
                        /* Store all relevant pixels in an array for sorting */
                        nb_good = 0 ;
                        for (k=0 ; k<nr ; k++) {
                            for (l=0 ; l<nc ; l++) {
                                im_pos = x + y * in->nx ;
                                if (x>=0 && x<in->nx && y>=0 && y<in->ny) {
                                    if (pbpm[im_pos]==CPL_BINARY_0) nb_good++ ;
                                }
                                /* Next col */
                                x++ ; 
                            }
                            /* Next row */
                            x -= nc ;
                            y-- ;
                        }
                        if (nb_good > 0) {
                            /* This pixel is reset as good */
                            pbpm_dil[curr_pos] = CPL_BINARY_0 ;
                            pix_arr = cpl_malloc(nb_good*sizeof(double)) ;
                            /* Go into upper left corner of current pixel   */
                            x = i - hsx ;
                            y = j + hsy ;
                            filt_pos = 0 ;
                            for (k=0 ; k<nr ; k++) {
                                for (l=0 ; l<nc ; l++) {
                                    im_pos = x + y * in->nx ;
                                    if (x>=0 && x<in->nx && y>=0 && y<in->ny) {
                                        if (pbpm[im_pos] == CPL_BINARY_0) {
                                            pix_arr[filt_pos] = 
                                                (double)pi[im_pos] ;
                                            filt_pos++ ; 
                                        }
                                    }
                                    /* Next col */
                                    x++ ; 
                                }
                                /* Next row */
                                x -= nc ;
                                y-- ;
                            }
                            /* Sort array with standard sorting routine */
                            cpl_tools_sort_double(pix_arr, nb_good) ;
                            /* Go into upper left corner of current pixel   */
                            x = i - hsx ;
                            y = j + hsy ;
                            filt_pos = 0 ;
                            sum_pix = 0.0 ;
                            for (k=0 ; k<nr ; k++) {
                                for (l=0 ; l<nc ; l++) {
                                    im_pos = x + y * in->nx ;
                                    if (x>=0 && x<in->nx && y>=0 && y<in->ny) {
                                        if (pbpm[im_pos] == CPL_BINARY_0) {
                                            sum_pix += ker_arr[filt_pos] * 
                                                pix_arr[filt_pos] ;
                                            ker_norm += fabs(ker_arr[filt_pos]);
                                            filt_pos++ ;
                                        }
                                    }
                                    /* Next col */
                                    x++ ; 
                                }
                                /* Next row */
                                x -= nc ;
                                y-- ;
                            }
                            cpl_free(pix_arr) ;

                            if (fabs(sum_pix) > FLT_MAX * ker_norm) {
                                /* Reject the pixel */
                                pbpm_dil[curr_pos] = CPL_BINARY_1;
                                po[curr_pos] = (CPL_TYPE) 0;
                            } else {
                                /* Normalize output */
                                /* Assign value to image_out */
                                po[curr_pos] = (CPL_TYPE) (sum_pix / ker_norm);
                            }

                        } else {
                            /* This pixel has only bad neighbours */
                            po[curr_pos] = (CPL_TYPE)0.0 ;
                        }
                    }
                }
            }
            break ;
          }

#elif defined CPL_OPERATION && CPL_OPERATION == CPL_IMAGE_FILTER_MEDIAN_NOBPM

        case CPL_TYPE_T:
        {
            const CPL_TYPE * pi;
            CPL_TYPE * po;
            CPL_TYPE pix_arr[nr * nc];

            out = cpl_image_new(in->nx, in->ny, CPL_TYPE_T);
            pi = (const CPL_TYPE*)in->pixels ;
            po = (CPL_TYPE*)out->pixels ;
            /* Main filter loop */
            for (j=hsy ; j<in->ny-hsy ; j++) {
                for (i=hsx ; i<in->nx-hsx ; i++) {
                    /* Curent pixel position */
                    curpos = i+j*in->nx ;
                    /* Go into upper left corner of current pixel */
                    im_pos = curpos - hsx + hsy*in->nx ;
                    filt_pos = pix_arr_pos = 0 ;
                    /* Store all relevant pixels in an array for sorting */
                    for (k=0 ; k<nr ; k++) {
                        for (l=0 ; l<nc ; l++) {
                            if (fabs(ker_arr[filt_pos]-1.0)<1e-5) {
                                pix_arr[pix_arr_pos] = pi[im_pos] ;
                                pix_arr_pos++ ;
                            }
                            filt_pos++ ;
                            im_pos++ ; 
                        }
                        /* Next row */
                        im_pos -= in->nx + nc ; 
                    }
                    /* find median by optimized median find */
                    po[curpos]=CPL_TYPE_MEDIAN(pix_arr, pix_arr_pos);
                }
            }
            break ;
          }

#elif defined CPL_OPERATION && CPL_OPERATION == CPL_IMAGE_FILTER_MEDIAN

        case CPL_TYPE_T:
        {
            const CPL_TYPE * pi = (const CPL_TYPE*)in->pixels;
            CPL_TYPE * po = (CPL_TYPE*)filtered->pixels;
            CPL_TYPE pix_arr[nr * nc];

            /* Main filter loop */
            for (j=0 ; j<in->ny ; j++) {
                for (i=0 ; i<in->nx ; i++) {
                    /* Curent pixel position */
                    curpos = i+j*in->nx ;
                    if (pbpm_dil[curpos] == CPL_BINARY_1) {
                        /* Go into upper left corner of current pixel   */
                        x = i - hsx ;
                        y = j + hsy ;
                        /* Store all relevant pixels in an array for sorting */
                        nb_good = 0 ;
                        for (k=0 ; k<nr ; k++) {
                            for (l=0 ; l<nc ; l++) {
                                im_pos = x + y * in->nx ;
                                if (x>=0 && x<in->nx && y>=0 && y<in->ny) {
                                    if ((pbpm[im_pos] == CPL_BINARY_0) && 
                                            (fabs(ker_arr[k+l*nr]-1.0)<1e-5)) {
                                        pix_arr[nb_good] = pi[im_pos] ;
                                        nb_good++ ;
                                    }
                                }
                                /* Next col */
                                x++ ; 
                            }
                            /* Next row */
                            x -= nc ;
                            y-- ;
                        }
                        if (nb_good > 0) {
                            /* This pixel is reset as good */
                            pbpm_dil[curpos] = CPL_BINARY_0 ;
                            /* find median */
                            po[curpos]=CPL_TYPE_MEDIAN(pix_arr, nb_good);
                        } else {
                            /* This pixel has only bad neighbours */
                            po[curpos]=(CPL_TYPE)0.0 ;
                        }
                    }
                }
            }
            break ;
          }

#elif defined CPL_OPERATION && CPL_OPERATION == CPL_IMAGE_FILTER_STDEV_NOBPM

        case CPL_TYPE_T:
        {
            CPL_TYPE * pi;
            CPL_TYPE * po;
            double variance;

            out = cpl_image_new(in->nx, in->ny, CPL_TYPE_T);
            pi = (CPL_TYPE*)in->pixels ;
            po = (CPL_TYPE*)out->pixels ;

            for (j=hsy ; j<in->ny-hsy ; j++) {
                /* Beginning of the line, compute the initial sum */
                i = hsx ;
                sum = 0.00 ;
                sq_sum = 0.00 ;
                for (l=j-hsy ; l<=j+hsy ; l++) {
                    for (k=i-hsx ; k<=i+hsx ; k++) {
                        pix = (double)pi[k+l*in->nx] ;
                        sum += pix ;
                        sq_sum += pix * pix ;
                    }
                }
                /* Round-off may cause a small variance to become negative */
                variance = sq_sum * inv_surf - sum*sum*inv_surf*inv_surf;
                po[i+j*in->nx] = (CPL_TYPE)(variance > 0 ? sqrt(variance) : 0);
                /* Compute the rest of the line using the initial sum */
                for (i=hsx+1 ; i<in->nx-hsx ; i++) {
                    /* Remove the previous first column */
                    k = i - hsx - 1 ;
                    for (l=j-hsy ; l<=j+hsy ; l++) {
                        pix = (double)pi[k+l*in->nx];
                        sum -= pix ;
                        sq_sum -= pix * pix ;
                    }
                    /* Add the new last column */
                    k = i + hsx ;
                    for (l=j-hsy ; l<=j+hsy ; l++) {
                        pix = (double)pi[k+l*in->nx];
                        sum += pix ;
                        sq_sum += pix * pix ;
                    }
                    /* Round-off may cause a small variance to become negative*/
                    variance = sq_sum * inv_surf - sum*sum*inv_surf*inv_surf;
                    po[i+j*in->nx] = (CPL_TYPE)(variance > 0 ? sqrt(variance) : 0);
                }
            }
            break ;
          }

#elif defined CPL_OPERATION && CPL_OPERATION == CPL_IMAGE_FILTER_STDEV

        case CPL_TYPE_T:
        {
            CPL_TYPE * pi;
            CPL_TYPE * po;

            pi = (CPL_TYPE*)in->pixels ;
            po = (CPL_TYPE*)filtered->pixels ;
            /* Main filter loop */
            for (j=0 ; j<in->ny ; j++) {
                for (i=0 ; i<in->nx ; i++) {
                    /* Curent pixel position */
                    curpos = i+j*in->nx ;
                    if (pbpm_dil[curpos] == CPL_BINARY_1) {
                        /* Go into upper left corner of current pixel   */
                        x = i - hsx ;
                        y = j + hsy ;
                        sum = 0.0 ;
                        sq_sum = 0.0 ;
                        inv_surf = 0.0 ;
                        /* Store all relevant pixels in an array for sorting */
                        for (k=0 ; k<nr ; k++) {
                            for (l=0 ; l<nc ; l++) {
                                im_pos = x + y * in->nx ;
                                if (x>=0 && x<in->nx && y>=0 && y<in->ny) {
                                    if (pbpm[im_pos] == CPL_BINARY_0) {
                                        /* This pixel is not bad any more */
                                        pbpm_dil[curpos] = CPL_BINARY_0 ; 
                                        pix = (double)pi[im_pos] ;
                                        sum += pix ;
                                        sq_sum += pix * pix ;
                                        inv_surf++ ;
                                    }
                                }
                                /* Next col */
                                x++ ; 
                            }
                            /* Next row */
                            x -= nc ;
                            y-- ;
                        }
                        if (inv_surf > 0) {
                            /* find stdev */
                            /* Round-off may cause a small variance to be <0 */
                            double variance = sq_sum/inv_surf
                                - sum*sum/(inv_surf*inv_surf);
                            po[curpos]=
                                (CPL_TYPE)(variance > 0 ? sqrt(variance) : 0);
                        }
                    }
                }
            }
            break ;
          }
#else

#ifdef ADDTYPE
#undef ADDTYPE
#endif
#define ADDTYPE(a) CONCAT2X(a,CPL_TYPE)


/*----------------------------------------------------------------------------*/
/**
   @internal
   @brief   Median filter
   @param   self   Image of type CPL_TYPE to filter
   @param   radx   x-radius of median window
   @param   rady   y-radius of median window
   @return  Filtered image, or NULL on error.
   @note copied from uves

   Apply a @em radx by @em rady median filter to the input image.
   Each pixel is replaced by the median of a local window of size (2* @em radx + 1)*
   (2 * @em rady + 1) centered on the pixel.

   
*/
/*----------------------------------------------------------------------------*/
static cpl_image *
ADDTYPE(cpl_image_filter_median_uniform)(const cpl_image * self, int radx,
                                         int rady)
{
    const CPL_TYPE * dself   = CPL_IMAGE_GET_DATA_CONST(self);
    cpl_image      * result  = cpl_image_new(self->nx, self->ny, self->type);
    CPL_TYPE       * dresult = CPL_IMAGE_GET_DATA(result);
    CPL_TYPE         window[(2*radx + 1) * (2*rady + 1)];
    int              x, y;

    /* A failure here indicates a serious bug in CPL,
       so don't care about the memory leak */
    cpl_ensure(cpl_image_get_type(self) == CPL_TYPE_T,
               CPL_ERROR_INCOMPATIBLE_INPUT, NULL);

    for (y = 0; y < self->ny; y++) {
        const int y_1 = CPL_IMAGE_FILTER_MAX(0,          y - rady);
        const int y2  = CPL_IMAGE_FILTER_MIN(self->ny-1, y + rady);
        for (x = 0; x < self->nx; x++) {
            const int x1 = CPL_IMAGE_FILTER_MAX(0,          x - radx);
            const int x2 = CPL_IMAGE_FILTER_MIN(self->nx-1, x + radx);
            int i, j;
            int k = 0;
            
            for (j = y_1; j <= y2; j++) {
                for (i = x1; i <= x2; i++) {
                    window[k++] = dself[i + j * self->nx];
                }
            }
            dresult[x + y * self->nx] = CPL_TYPE_MEDIAN(window, k);
        }
    }
    return result;
}

#endif

#undef CPL_TYPE
#undef CPL_TYPE_T
#undef CPL_TYPE_MEDIAN
#undef CPL_IMAGE_GET_DATA
#undef CPL_IMAGE_GET_DATA_CONST
