/* $Id: cpl_imagelist_basic_body.h,v 1.25 2008/07/21 08:48:40 llundin Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2008 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Type dependent macros */
#if defined CPL_CLASS && CPL_CLASS == CPL_CLASS_DOUBLE
#define CPL_TYPE double
#define CPL_TYPE_T CPL_TYPE_DOUBLE
#define CPL_IMAGE_GET_DATA cpl_image_get_data_double
#define CPL_IMAGE_GET_DATA_CONST cpl_image_get_data_double_const
#define CPL_IMAGE_GET_MEDIAN cpl_tools_get_median_double

#elif defined CPL_CLASS && CPL_CLASS == CPL_CLASS_FLOAT
#define CPL_TYPE float
#define CPL_TYPE_T CPL_TYPE_FLOAT
#define CPL_IMAGE_GET_DATA cpl_image_get_data_float
#define CPL_IMAGE_GET_DATA_CONST cpl_image_get_data_float_const
#define CPL_IMAGE_GET_MEDIAN cpl_tools_get_median_float

#elif defined CPL_CLASS && CPL_CLASS == CPL_CLASS_INT
#define CPL_TYPE int
#define CPL_TYPE_T CPL_TYPE_INT
#define CPL_IMAGE_GET_DATA cpl_image_get_data_int
#define CPL_IMAGE_GET_DATA_CONST cpl_image_get_data_int_const
#define CPL_IMAGE_GET_MEDIAN cpl_tools_get_median_int

#else
#undef CPL_TYPE
#undef CPL_TYPE_T
#undef CPL_IMAGE_GET_DATA
#undef CPL_IMAGE_GET_MEDIAN
#endif

#if CPL_OPERATION == CPL_IMLIST_BASIC_OPER

    int i ;

    /* Check input image sets */
    cpl_ensure_code(in1, CPL_ERROR_NULL_INPUT);
    cpl_ensure_code(in2, CPL_ERROR_NULL_INPUT) ;

    /* Check image sets compatibility     */
    cpl_ensure_code( in1->ni == in2->ni, CPL_ERROR_ILLEGAL_INPUT);

    /* Loop on the planes and apply the operation */
    for (i=0 ; i<in1->ni ; i++) {
        const cpl_error_code error = 
            CPL_OPERATOR(in1->images[i], in2->images[i]);
        cpl_ensure_code(!error, error);
    }

    return CPL_ERROR_NONE ;

#elif CPL_OPERATION == CPL_IMLIST_BASIC_IMAGE_LOCAL

    int i ;

    /* Check input image sets */
    cpl_ensure_code(imlist, CPL_ERROR_NULL_INPUT);
    cpl_ensure_code(img,    CPL_ERROR_NULL_INPUT) ;

    /* Loop on the planes and apply the operation */
    for (i=0 ; i<imlist->ni ; i++) {
        const cpl_error_code error = CPL_OPERATOR(imlist->images[i], img);
        cpl_ensure_code(!error, error);
    }

    return CPL_ERROR_NONE ;

#elif CPL_OPERATION == CPL_IMLIST_BASIC_TIME_MEDIAN
#ifndef L2_CACHE_BYTES
/* The size in bytes of the cache level, with most importance 
   (probably related to size times penalty for a miss) */
/* FIXME: Assume 256kB (L2) Cache. Many CPUs have more (e.g. 512kB),
          but the performance on those does not seem to suffer much */
#define L2_CACHE_BYTES 262144
#endif
    case CPL_TYPE_T:
    {
        const int nz = imlist->ni;
        CPL_TYPE ** pi = cpl_malloc(nz * sizeof(CPL_TYPE*));
        CPL_TYPE *  po = CPL_IMAGE_GET_DATA(median);
        CPL_TYPE *  timeline = NULL;

        for (k=0; k < nz; k++)
            pi[k] = CPL_IMAGE_GET_DATA(imlist->images[k]);

#if L2_CACHE_BYTES > 0
        if (L2_CACHE_BYTES > 0 &&
            sizeof(CPL_TYPE) * nz * (1 + nx * ny) > L2_CACHE_BYTES) {
            /* There is a cache available, and the entire imagelist does not
               fit inside it */

            /* Heuristics: When there are just a few planes, remapping does not
               pay off. It seems that break-even is at nz = nx / 15. */
            const int max_aspect = 15;

            /* The largest number of pixels that can be handled by the cache */
            const int mxl2 = (L2_CACHE_BYTES - sizeof(CPL_TYPE *) * nz)
                / (sizeof(CPL_TYPE) * (1 + 2 * nz));
            const int mxmax = mxl2 < nz * max_aspect ? mxl2 : nz * max_aspect;
            int mx = nx;
            int my = ny;


            while ((mx & 1) == 0 && mx > mxmax) {
                /* Make mx smaller, until it fits */
                mx >>= 1;
                my <<= 1;
            }

            if (mx == nx ) {
                /* Did not yet change mx, try to make it larger while it fits */
                while ((mx<<1) < mxmax && (my & 1) == 0) {
                    mx <<=1;
                    my >>=1;
                }
            }


            if (sizeof(CPL_TYPE *) * nz + sizeof(CPL_TYPE) * mx * (1 + 2 * nz)
              > L2_CACHE_BYTES/2) {
                /* Heuristics: Too few planes may cause only a fraction of the
                   cache to be usable, in this case fall back on the simple
                   method. It seems that the break-even is about 50% cache
                   usage */
                timeline = cpl_malloc(mx * nz * sizeof(CPL_TYPE));

                /* For each time line */
                for (j=0; j < my; j++) {
                    /* Get the pixels on the current time line  */
                    for (k=0; k < nz; k++)
                        for (i=0; i < mx; i++)
                            timeline[k + i * nz] = pi[k][i + j * mx];

                    for (i=0; i < mx; i++)
                        /* Compute the median */
                        po[i + j * mx] = CPL_IMAGE_GET_MEDIAN(timeline
                                                              + i * nz,
                                                              nz);
                }
            }
        }
#endif
        if (timeline == NULL)  {
            /* Remap one pixel at a time */
            timeline = cpl_malloc(nz * sizeof(CPL_TYPE));
            /* For each time line */
            for (j=0 ; j < ny ; j++) {
                for (i=0 ; i < nx ; i++) {
                    /* Get the pixels on the current time line  */
                    for (k=0 ; k < nz ; k++)
                        timeline[k] = pi[k][i + j * nx] ;

                    /* Compute the median */
                    po[i + j * nx] = CPL_IMAGE_GET_MEDIAN(timeline, nz);
                }
            }
        }

        cpl_free(pi);
        cpl_free(timeline);
        break;
    }

#elif CPL_OPERATION == CPL_IMLIST_BASIC_TIME_MINMAX

    case CPL_TYPE_T:
    {
        CPL_TYPE       * pout_ima ;
        const CPL_TYPE * pcur_ima ;
        
        out_ima = cpl_image_new(nx, ny, CPL_TYPE_T) ;
        pout_ima = CPL_IMAGE_GET_DATA(out_ima) ;

        /* Loop on the pixels */
        for (j=0 ; j<ny ; j++) {
            for (i=0 ; i<nx ; i++) {
                /* Fill the vector */
                for (k=0 ; k<ni ; k++) {
                    cur_ima = cpl_imagelist_get_const(imlist, k) ;
                    pcur_ima = CPL_IMAGE_GET_DATA_CONST(cur_ima) ;
                    ptime_line[k] = (double)pcur_ima[i+j*nx] ;
                }
                /* Sort the time line */
                cpl_vector_sort(time_line, 1) ;
                    
                /* Set high and low to 0 */
                for (l=0 ; l<low ; l++) ptime_line[l] = 0.0 ;
                for (l=ni-high ; l<ni ; l++) ptime_line[l] = 0.0 ;

                /* Compute the average */
                out_val = cpl_vector_get_mean(time_line) ;
                pout_ima[i+j*nx] = (CPL_TYPE)(out_val * ni / (ni-low-high)) ;
            }
        }
        break ;
    }

#elif CPL_OPERATION == CPL_IMLIST_BASIC_TIME_SIGCLIP

    case CPL_TYPE_T:
    {
        CPL_TYPE       * pout_ima ;
        const CPL_TYPE * pcur_ima ;
        
        out_ima = cpl_image_new(nx, ny, CPL_TYPE_T) ;
        pout_ima = CPL_IMAGE_GET_DATA(out_ima) ;

        /* Loop on the pixels */
        for (j=0 ; j<ny ; j++) {
            for (i=0 ; i<nx ; i++) {
                /* Fill the vector */
                for (k=0 ; k<ni ; k++) {
                    cur_ima = cpl_imagelist_get_const(imlist, k) ;
                    pcur_ima = CPL_IMAGE_GET_DATA_CONST(cur_ima) ;
                    ptime_line[k] = (double)pcur_ima[i+j*nx] ;
                }
                /* Get the mean and stdev */
                mean  = cpl_vector_get_mean(time_line) ;
                stdev = cpl_vector_get_stdev(time_line) ;
                low_thresh = mean - sigma * stdev ;
                high_thresh = mean + sigma * stdev ;

                /* Compute the final value */
                out_val = 0.0 ;
                nb_val = 0 ;
                for (k=0 ; k<ni ; k++) {
                    if (ptime_line[k]<=high_thresh && 
                        ptime_line[k]>=low_thresh) {
                        out_val += (double)ptime_line[k] ;
                        nb_val ++ ;
                    } 
                }
                if (nb_val>0) out_val /= nb_val ;

                /* Set the output image pixel value */
                pout_ima[i+j*nx] = (CPL_TYPE)out_val ;
            }
        }
        break ;
    }

#elif CPL_OPERATION == CPL_IMLIST_BASIC_SWAP_AXIS 

    case CPL_TYPE_T:
    {
        CPL_TYPE       * pcur_ima ;
        const CPL_TYPE * pold_ima ;

        /* SWAP X <-> Z */
        if (mode == CPL_SWAP_AXIS_XZ) {
            for (i=0 ; i<nx ; i++) {
                cur_ima=cpl_image_new(ni, ny, CPL_TYPE_T) ;
                pcur_ima = CPL_IMAGE_GET_DATA(cur_ima) ;
                for (j=0 ; j<ni ; j++) {
                    old_ima = cpl_imagelist_get_const(ilist, j) ;
                    pold_ima = CPL_IMAGE_GET_DATA_CONST(old_ima) ;
                    for (k=0 ; k<ny ; k++) {
                        pcur_ima[j+k*ni] = pold_ima[i+k*nx] ;
                    }
                }
                cpl_imagelist_set(swapped, cur_ima, i) ;
            }
        } else {
        /* SWAP Y <-> Z */
            for (i=0 ; i<ny ; i++) {
                cur_ima=cpl_image_new(nx, ni, CPL_TYPE_T) ;
                pcur_ima = CPL_IMAGE_GET_DATA(cur_ima) ;
                for (j=0 ; j<ni ; j++) {
                    old_ima = cpl_imagelist_get_const(ilist, j) ;
                    pold_ima = CPL_IMAGE_GET_DATA_CONST(old_ima) ;
                    for (k=0 ; k<nx ; k++) {
                        pcur_ima[k+j*nx] = pold_ima[k+i*nx] ;
                    }
                }
                cpl_imagelist_set(swapped, cur_ima, i) ;
            }
        }
        break ;
    }

#endif

#undef CPL_TYPE
#undef CPL_TYPE_T
#undef CPL_IMAGE_GET_DATA
#undef CPL_IMAGE_GET_DATA_CONST
#undef CPL_IMAGE_GET_MEDIAN
