/* $Id: cpl_io.h,v 1.9 2008/07/21 08:48:39 llundin Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2008 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: llundin $
 * $Date: 2008/07/21 08:48:39 $
 * $Revision: 1.9 $
 * $Name: cpl-4_2_0b3 $
 */

#ifndef CPL_IO_H
#define CPL_IO_H

/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include <cpl_macros.h>

CPL_BEGIN_DECLS

/*-----------------------------------------------------------------------------
                                   Defines
 -----------------------------------------------------------------------------*/
/**
 * Each FITS IO mode is listed below.
 * The modes can be combined with bitwise or.
 */

/* The default FITS IO mode.
 * In this mode files opened for writing will be overwritten
 * CPL_IO_DEFAULT is unneccesary together with other IO modes
 */
#define CPL_IO_DEFAULT ((unsigned)0)

/* FITS append mode */
#define CPL_IO_EXTEND  ((unsigned)1 << 0)

/* The maximum value for the IO mode bitmask */
#define CPL_IO_MAX     (CPL_IO_EXTEND)

typedef enum {
    /* FITS pixel depths */

    /**
     * FITS BITPIX=8
     * @hideinitializer
     */
    CPL_BPP_8_UNSIGNED = 8,

    /**
     * FITS BITPIX=16
     * @hideinitializer
     */
    CPL_BPP_16_SIGNED = 16,

    /**
     * FITS BITPIX=16
     * FITS BZERO=32768
     * @hideinitializer
     */
    CPL_BPP_16_UNSIGNED = -16,

    /**
     * FITS BITPIX=32
     * @hideinitializer
     */
    CPL_BPP_32_SIGNED = 32,

    /**
     * FITS BITPIX=-32
     * @hideinitializer
     */
    CPL_BPP_IEEE_FLOAT = -32,

    /**
     * FITS BITPIX=-64
     * @hideinitializer
     */
    CPL_BPP_IEEE_DOUBLE = -64

} cpl_type_bpp;

CPL_END_DECLS

#endif /* CPL_IO_H */
