/* $Id: cpl_stats_body.h,v 1.10 2008/07/21 08:48:40 llundin Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2008 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Type dependent macros */
#if CPL_CLASS == CPL_CLASS_DOUBLE
#define CPL_TYPE double
#define CPL_TYPE_T CPL_TYPE_DOUBLE
#define CPL_ADD_FLOPS cpl_tools_add_flops

#elif CPL_CLASS == CPL_CLASS_FLOAT
#define CPL_TYPE float
#define CPL_TYPE_T CPL_TYPE_FLOAT
#define CPL_ADD_FLOPS cpl_tools_add_flops

#elif CPL_CLASS == CPL_CLASS_INT
#define CPL_TYPE int
#define CPL_TYPE_T CPL_TYPE_INT
#define CPL_ADD_FLOPS(N) /* N integer ops */

#else
#undef CPL_TYPE
#undef CPL_TYPE_T
#endif


#if CPL_OPERATION == CPL_IMAGE_STATS_ALL

  case CPL_TYPE_T:
  {
      const CPL_TYPE * pi = (const CPL_TYPE*)image->pixels;
 
      min_pos = max_pos = firstgoodpos;
      min_pix = (double)pi[firstgoodpos] ;
      max_pix = (double)pi[firstgoodpos] ;

      for (j=lly-1 ; j<ury ; j++) {
          pos = (llx-1)+j*image->nx ;

          for (i=llx-1 ; i<urx ; i++) {
              if (badmap == NULL || !badmap[pos]) {
                  const double delta = (double)pi[pos] - pix_mean;

                  pix_var  += ipix * delta * delta / (ipix + 1.0);
                  pix_mean += delta / (ipix + 1.0);
                  ipix += 1.0;

                  if (pi[pos] < min_pix) min_pix = (double)pi[min_pos = pos];
                  if (pi[pos] > max_pix) max_pix = (double)pi[max_pos = pos];

                  pix_sum += (double)pi[pos];
                  abs_sum += fabs((double)pi[pos]);
                  sqr_sum += (double)pi[pos] * (double)pi[pos];
              }
              pos++;
          }
      }
      CPL_ADD_FLOPS(14 * npix);
      break ;
  }

#elif CPL_OPERATION == CPL_IMAGE_STATS_VARIANCE

  case CPL_TYPE_T:
  {
      const CPL_TYPE * pi = (const CPL_TYPE*)image->pixels;

      for (j=lly-1 ; j<ury ; j++) {
          pos = (llx-1)+j*image->nx ;
          for (i=llx-1 ; i<urx ; i++) {
              if (badmap == NULL || !badmap[pos]) {
                  const double delta = (double)pi[pos] - pix_mean;

                  /* The round-off on pix_mean may be different here
                     and in the all-stats block :-( */
                  pix_var  += ipix * delta * delta / (ipix + 1.0);
                  pix_mean += delta / (ipix + 1.0);
                  ipix += 1.0;

              }
              pos++;
          }
      }
      CPL_ADD_FLOPS(7 * npix);
      break ;
  }

#elif CPL_OPERATION == CPL_IMAGE_STATS_CENTROID

  case CPL_TYPE_T:
  {
      const CPL_TYPE * pi = (const CPL_TYPE*)image->pixels;
      const double min_pix_tmp = min_pix < 0.0 ? min_pix : 0.0;

      for (j=lly-1 ; j<ury ; j++) {
          pos = (llx-1)+j*image->nx ;
          for (i=llx-1 ; i<urx ; i++) {
              if (badmap == NULL || !badmap[pos]) {
                  sum_xz += ((double)pi[pos]-min_pix_tmp)*(double)(i+1) ;
                  sum_yz += ((double)pi[pos]-min_pix_tmp)*(double)(j+1) ;
                  sum_z += (double)pi[pos]-min_pix_tmp ;
                  sum_x  += (double)(i+1) ;
                  sum_y  += (double)(j+1) ;
              }
              pos++;
          }
      }
      if (sum_z  < 0) sum_z  = 0; /* Can only become negative due to rounding */
      if (sum_xz < 0) sum_xz = 0; /* Can only become negative due to rounding */
      if (sum_yz < 0) sum_yz = 0; /* Can only become negative due to rounding */
      CPL_ADD_FLOPS(8 * npix);
      break ;
  }

#elif CPL_OPERATION == CPL_IMAGE_STATS_MINMAX

  case CPL_TYPE_T:
  {
      const CPL_TYPE * pi = (const CPL_TYPE*)image->pixels;
      /* Avoid in-loop casting */
      CPL_TYPE min_tmp = pi[firstgoodpos];
      CPL_TYPE max_tmp = pi[firstgoodpos];

      min_pos = max_pos = firstgoodpos;
      for (j=lly-1 ; j<ury ; j++) {
          pos = (llx-1)+j*image->nx ;

          for (i=llx-1 ; i<urx ; i++) {
              if (badmap == NULL || !badmap[pos]) {
                 if (pi[pos] < min_tmp) min_tmp = pi[min_pos = pos];
                 if (pi[pos] > max_tmp) max_tmp = pi[max_pos = pos];
              }
              pos++;
          }
      }
      min_pix = (double)min_tmp;
      max_pix = (double)max_tmp;
      CPL_ADD_FLOPS(2 * npix);
      break ;
  }

#elif CPL_OPERATION == CPL_IMAGE_STATS_FLUX

  case CPL_TYPE_T:
  {
      const CPL_TYPE * pi = (const CPL_TYPE*)image->pixels;

      for (j=lly-1 ; j<ury ; j++) {
          pos = (llx-1)+j*image->nx ;

          for (i=llx-1 ; i<urx ; i++) {
              if (badmap == NULL || !badmap[pos]) {
                  pix_sum += (double)pi[pos];
                  abs_sum += fabs((double)pi[pos]);
                  sqr_sum += (double)pi[pos] * (double)pi[pos];
              }
              pos++;
          }
      }

      CPL_ADD_FLOPS(5 * npix);
      break ;
  }

#endif

#undef CPL_TYPE
#undef CPL_TYPE_T
#undef CPL_ADD_FLOPS
