/* $Id: cpl_type.h,v 1.8 2005/11/21 10:53:33 yjung Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2005 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: yjung $
 * $Date: 2005/11/21 10:53:33 $
 * $Revision: 1.8 $
 * $Name: cpl-4_2_0b3 $
 */

#ifndef CPL_TYPE_H
#define CPL_TYPE_H

#include <cpl_macros.h>


CPL_BEGIN_DECLS

/**
 * @ingroup cpl_type
 *
 * @brief
 *   The CPL type codes and flags.
 */

enum _cpl_type_ {

    /* flags */

    /**
     * Flag indicating whether a type is an array or a basic type.
     * @hideinitializer
     */
    CPL_TYPE_FLAG_ARRAY   = 1 << 0,

    /* Padded for future extensions */

    /* types */

    /**
     * Invalid or undetermined type.
     * @hideinitializer
     */
    CPL_TYPE_INVALID = 1 << 4,

    /**
     * Type code corresponding to type @c char.
     * @hideinitializer
     */
    CPL_TYPE_CHAR    = 1 << 5,

    /**
     * Type code corresponding to type @c unsigned char.
     * @hideinitializer
     */
    CPL_TYPE_UCHAR   = 1 << 6,

    /**
     * Type code corresponding to the boolean type.
     * @hideinitializer
     */
    CPL_TYPE_BOOL    = 1 << 7,

    /**
     * Type code corresponding to type @c int.
     * @hideinitializer
     */
    CPL_TYPE_INT     = 1 << 8,

    /**
     * Type code corresponding to type @c unsigned int.
     * @hideinitializer
     */
    CPL_TYPE_UINT    = 1 << 9,

    /**
     * Type code corresponding to type @c long.
     * @hideinitializer
     */
    CPL_TYPE_LONG    = 1 << 10,

    /**
     * Type code corresponding to type @c unsigned long.
     * @hideinitializer
     */
    CPL_TYPE_ULONG   = 1 << 11,

    /**
     * Type code corresponding to type @c float.
     * @hideinitializer
     */
    CPL_TYPE_FLOAT   = 1 << 12,

    /**
     * Type code corresponding to type @c double.
     * @hideinitializer
     */
    CPL_TYPE_DOUBLE  = 1 << 13,

    /**
     * Type code corresponding to a pointer type.
     * @hideinitializer
     */
    CPL_TYPE_POINTER = 1 << 14,

    /**
     * Type code corresponding to a character array.
     * @hideinitializer
     */
    CPL_TYPE_STRING  = (CPL_TYPE_CHAR | CPL_TYPE_FLAG_ARRAY)

};


/**
 * @ingroup cpl_type
 *
 * @brief
 *   The type code type.
 */

typedef enum _cpl_type_ cpl_type;


enum _cpl_boolean_ {
    CPL_FALSE = 0,
    CPL_TRUE = !CPL_FALSE
};

typedef enum _cpl_boolean_ cpl_boolean;


size_t cpl_type_get_sizeof(cpl_type type);

CPL_END_DECLS

#endif /* CPL_TYPE_H */
