/* $Id: cpl_apertures.h,v 1.2 2008/07/21 08:48:40 llundin Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2008 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: llundin $
 * $Date: 2008/07/21 08:48:40 $
 * $Revision: 1.2 $
 * $Name: cpl-4_2_0b3 $
 */

#ifndef CPL_APERTURES_H
#define CPL_APERTURES_H

/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include <stdio.h>
#include <cpl_vector.h>
#include <cpl_image.h>

CPL_BEGIN_DECLS

/*-----------------------------------------------------------------------------
                                   New types
 -----------------------------------------------------------------------------*/

typedef struct _cpl_apertures_ cpl_apertures ;

/*-----------------------------------------------------------------------------
                            Function prototypes
 -----------------------------------------------------------------------------*/

/* IO functions */
cpl_apertures * cpl_apertures_new(int) ;
void cpl_apertures_delete(cpl_apertures *) ;
void cpl_apertures_dump(const cpl_apertures *, FILE *) ;

/* Statistics computation */
cpl_apertures * cpl_apertures_new_from_image(const cpl_image *, const cpl_image *) ;

/* Accessor functions */
int cpl_apertures_get_size(const cpl_apertures *) ;
double cpl_apertures_get_max_x(const cpl_apertures *, int) ;
double cpl_apertures_get_max_y(const cpl_apertures *, int) ;
double cpl_apertures_get_centroid_x(const cpl_apertures *, int) ;
double cpl_apertures_get_centroid_y(const cpl_apertures *, int) ;
int cpl_apertures_get_npix(const cpl_apertures *, int) ;
int cpl_apertures_get_left(const cpl_apertures *, int) ;
int cpl_apertures_get_left_y(const cpl_apertures *, int) ;
int cpl_apertures_get_right(const cpl_apertures *, int) ;
int cpl_apertures_get_right_y(const cpl_apertures *, int) ;
int cpl_apertures_get_top_x(const cpl_apertures *, int) ;
int cpl_apertures_get_top(const cpl_apertures *, int) ;
int cpl_apertures_get_bottom_x(const cpl_apertures *, int) ;
int cpl_apertures_get_bottom(const cpl_apertures *, int) ;
double cpl_apertures_get_max(const cpl_apertures *, int) ;
double cpl_apertures_get_min(const cpl_apertures *, int) ;
double cpl_apertures_get_mean(const cpl_apertures *, int) ;
double cpl_apertures_get_median(const cpl_apertures *, int) ;
double cpl_apertures_get_stdev(const cpl_apertures *, int) ;
double cpl_apertures_get_flux(const cpl_apertures *, int) ;

/* Sorting functions */
cpl_error_code cpl_apertures_sort_by_npix(cpl_apertures *);
cpl_error_code cpl_apertures_sort_by_max(cpl_apertures  *);
cpl_error_code cpl_apertures_sort_by_flux(cpl_apertures *);

/* Detection functions */
cpl_apertures * cpl_apertures_extract(const cpl_image *, const cpl_vector *,
                                      int *);
cpl_apertures * cpl_apertures_extract_window(const cpl_image *,
                                             const cpl_vector *, int, int,
                                             int, int, int *);

cpl_apertures * cpl_apertures_extract_sigma(const cpl_image *, double) ;

CPL_END_DECLS

#endif 
