/* $Id: cpl_phys_const.h,v 1.6 2008/07/21 08:48:40 llundin Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2008 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: llundin $
 * $Date: 2008/07/21 08:48:40 $
 * $Revision: 1.6 $
 * $Name: cpl-4_2_0b3 $
 */

#ifndef CPL_PHYS_CONST_H
#define CPL_PHYS_CONST_H

CPL_BEGIN_DECLS

/*-----------------------------------------------------------------------------
                                Defines
 -----------------------------------------------------------------------------*/

/* Fundamental physical constants from http://physics.nist.gov/constants/
   (the above page has been moved, probably to http://physics.nist.gov/cuu/)
   SI-units are used unless otherwise stated.

   The constants are listed in numerical order
*/

/* Planck constant [Js] */
#define CPL_PHYS_H 6.6260693E-34 

/* Boltzmann constant [J/K] */
#define CPL_PHYS_K 1.3806505E-23

/* Wien displacement law constant [mK] (meter * Kelvin)*/
#define CPL_PHYS_Wien 2.8977685E-3

/* The speed of light in vacuum [m/s] */
#define CPL_PHYS_C 299792458

/* These dimension definitions will be replaced ASAP */
enum _cpl_unit_ {
    CPL_UNIT_LESS           =  1,  /* Dimension-less */
    CPL_UNIT_RADIAN         =  2,  /* [radian] */
    CPL_UNIT_LENGTH         =  3,  /* [m] */
    CPL_UNIT_FREQUENCY      =  5,  /* [1/s] */
    CPL_UNIT_PHOTONRADIANCE =  7,  /* [  radian/s/m^3] */
    CPL_UNIT_ENERGYRADIANCE = 11   /* [J*radian/s/m^3] */
};

typedef enum _cpl_unit_ cpl_unit;

CPL_END_DECLS

#endif

